/*==================================================
 *  XmlHttp Utility Functions
 *==================================================
 */

function XmlHttp()
{
    // empty constructor
}


/**
 *  Callback for XMLHttp onRequestStateChange.
 */
XmlHttp.s_OnReadyStateChange = function(XmlHttpObj, url, xmlhttp, fError, fDone)
{
    var kState_Complete = 4;
    var kStatus_OK = 200;

    var null_func = function() {};
    if (!fDone) {
        fDone = null_func;
    }
    if (!fError) {
        fError = null_func;
    }

    switch (xmlhttp.readyState) {
    // 1: Request not yet made
    // 2: Contact established with server but nothing downloaded yet
    // 3: Called multiple while downloading in progress
    
    // Download complete
    case kState_Complete:
        try {
            if (xmlhttp.status == 0     // file:// urls, works on Firefox
                || xmlhttp.status == kStatus_OK   // http:// urls
               ) {
                fDone(xmlhttp);
                XmlHttpObj.x_RequestCompleted(url, xmlhttp);
            }
            else {
                fError(
                    xmlhttp.statusText,
                    xmlhttp.status,
                    xmlhttp
                );
                XmlHttpObj.x_RequestFailed(url);
            }
        }
        catch (e) {
            //Debug.exception(e);
            fError(
                e,
                xmlhttp.status,
                xmlhttp
            );
            XmlHttpObj.x_RequestFailed(url);
        }
        break;
    }
};

/**
 *  Creates an XMLHttpRequest object. On the first run, this
 *  function creates a platform-specific function for
 *  instantiating an XMLHttpRequest object and then replaces
 *  itself with that function.
 */
XmlHttp.x_CreateRequest = function()
{
    if (window.ActiveXObject) {
        var programIDs = [
        "Msxml2.XMLHTTP",
        "Microsoft.XMLHTTP",
        "Msxml2.XMLHTTP.4.0"
        ];
        for (var i = 0; i < programIDs.length; i++) {
            try {
                var programID = programIDs[i];
                var f = function() {
                    return new ActiveXObject(programID);
                };
                var o = f();
                
                // Replace the x_CreateRequest function with a new
                // (inner) function. Therefore, the contidional logic
                // is only executed the first time.
                XmlHttp.x_CreateRequest = f;
                
                return o;
            } catch (e) {
                // silent
            }
        }
        throw new Error("Failed to create an XMLHttpRequest object");
    }
    else {
        try {
            var f = function() {
                return new XMLHttpRequest();
            };
            var o = f();
            
            // Replace the x_CreateRequest function with a new
            // (inner) function. Therefore, the contidional logic
            // is only executed the first time.
            XmlHttp.x_CreateRequest = f;
            
            return o;
        } catch (e) {
            throw new Error("Failed to create an XMLHttpRequest object");
        }
    }
};

XmlHttp.prototype.x_ProcessRequest = function(method, url, content, fError, fDone)
{
    var xmlhttp = XmlHttp.x_CreateRequest();
    var xmlhttpobj = this;
    
    xmlhttp.open(method, url, true);
    xmlhttp.onreadystatechange = function()
    {
        XmlHttp.s_OnReadyStateChange(xmlhttpobj, url, xmlhttp, fError, fDone);
    };

    if (method == "POST") {
        xmlhttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    }

    xmlhttp.send(content);

    return xmlhttp;
};



XmlHttp.prototype.x_RequestCompleted = function(url, xmlhttp)
{
};


XmlHttp.prototype.x_RequestFailed = function(url)
{
    alert("Request Failed: " + url);
};


/**
 *  Performs an asynchronous HTTP GET.
 *  fError is of the form function(statusText, statusCode, xmlhttp).
 *  fDone is of the form function(xmlhttp).
 */
XmlHttp.prototype.Get = function(url, fError, fDone)
{
    var xmlhttp = this.x_ProcessRequest("GET", url, null, fError, fDone);
};


/**
 *  Performs an asynchronous HTTP POST.
 *  fError is of the form function(statusText, statusCode, xmlhttp).
 *  fDone is of the form function(xmlhttp).
 */
XmlHttp.prototype.Post = function(url, content, fError, fDone)
{
    var xmlhttp = this.x_ProcessRequest("POST", url, content, fError, fDone);
};


XmlHttp.ForceXML = function(xmlhttp)
{
    try {
        xmlhttp.overrideMimeType("text/xml");
    }
    catch (e) {
        //xmlhttp.setRequestHeader("Content-Type", "text/xml");
    }
};

